package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.opt.dao.mapper.CrossInfoMapper;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.trend.CrossDataRealtimeMapper;
import net.wanji.opt.dto.OptDataExtend;
import net.wanji.opt.po.base.CrossInfoPO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.trend.CrossDataRealtimePO;
import net.wanji.opt.service.EvaluateService;
import net.wanji.opt.vo.EvaluateCrossDetailVO;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/3/10 10:47
 */
@Service
@Slf4j
public class EvaluateServiceImpl implements EvaluateService {
    private final CrossInfoMapper crossInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;

    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();

    public EvaluateServiceImpl(CrossInfoMapper crossInfoMapper, CrossSchemeOptLogMapper crossSchemeOptLogMapper, CrossDataRealtimeMapper crossDataRealtimeMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
    }

    @Override
    public EvaluateCrossDetailVO evaluateCrossDetail(CrossIdDTO crossIdDTO) {
        EvaluateCrossDetailVO evaluateCrossDetailVO = new EvaluateCrossDetailVO();
        String crossId = crossIdDTO.getCrossId();
        CrossInfoPO crossInfoPO = crossInfoMapper.selectById(crossId);
        String crossName = crossInfoPO.getName();
        evaluateCrossDetailVO.setCrossName(crossName);

        fillOptData(evaluateCrossDetailVO, crossId);
        fillRealtimeData(evaluateCrossDetailVO, crossId);

        return evaluateCrossDetailVO;
    }

    private void fillRealtimeData(EvaluateCrossDetailVO evaluateCrossDetailVO, String crossId) {
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        evaluateCrossDetailVO.setSceneStartTime(crossDataRealtimePO.getStartTime());
        evaluateCrossDetailVO.setDuration(crossDataRealtimePO.getDuration());
        // todo 服务水平定义
        evaluateCrossDetailVO.setServiceLevel("D");
        evaluateCrossDetailVO.setFlow(crossDataRealtimePO.getFlow());
        evaluateCrossDetailVO.setLength(crossDataRealtimePO.getQueueLength());
        evaluateCrossDetailVO.setStopTimes(crossDataRealtimePO.getStopTimes());
    }

    private void fillOptData(EvaluateCrossDetailVO evaluateCrossDetailVO, String crossId) {
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        if (CollectionUtil.isEmpty(maxList)) {
            throw new RuntimeException("没有该路口优化日志数据");
        }
        CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
        // 解析dataExtend
        String dataExtend = crossSchemeOptLogPO.getDataExtend();
        OptDataExtend optDataExtend = gson.fromJson(dataExtend, OptDataExtend.class);
        evaluateCrossDetailVO.setSceneName(optDataExtend.getSceneName());
        evaluateCrossDetailVO.setStrategyName(optDataExtend.getStrategyName());
        evaluateCrossDetailVO.setIdeaName(optDataExtend.getIdeaName());
        evaluateCrossDetailVO.setOptStartTime(crossSchemeOptLogPO.getStartTime());
        Date endTime = crossSchemeOptLogPO.getEndTime();
        if (ObjectUtil.isEmpty(endTime)) {
            evaluateCrossDetailVO.setOptStatus(2);
        } else {
            evaluateCrossDetailVO.setOptStatus(1);
        }
    }
}
